/*
	CodeIT Round 1 (2011 - 2012)
	Copyright @2011 Ivan Bobev
*/

#define _CRT_SECURE_NO_WARNINGS

#include <iostream>
#include <vector>
#include <stack>
#include <map>
#include <set>
#include <cstdio>
#include <ctime>
#include <cassert>

using namespace std;

const int MAX_BOARD_SIZE = 350;

struct Node {
	int size;
	short color;
	set<int> neighbours;
};

typedef map<int, Node> Graph;

int n, c;
short board[MAX_BOARD_SIZE][MAX_BOARD_SIZE];
bool visited[MAX_BOARD_SIZE][MAX_BOARD_SIZE];
int shift[4][2] = {{-1, 0}, {1, 0}, {0, -1}, {0, 1}};
vector<int> connectedComponents;
int position, bestPosition;
vector<short> moves, bestMoves;
set<int> choosedStartPositions;
Graph graph, _graph;
clock_t maxTime;

void readInput() {
	FILE* finp = fopen("flood-it.in", "r");
	fscanf(finp, "%d %d", &n, &c);
	for (int i = 0; i < n * n; ++i)
		fscanf(finp, "%d", &board[i / n][i % n]);
	fclose(finp);
}

void printInput() {
	cout << n << " " << c << endl;
	for (int i = 0; i < n; ++i) {
		for (int j = 0; j < n; ++j)
			cout << board[i][j] << " ";
		cout << endl;
	}
}

int encode(int i, int j) {
	return i * n + j;
}

void findConnectedComponents() {
	connectedComponents.resize(n * n);
	for (int i = 0; i < n; ++i)
		for (int j = 0; j < n; ++j)
			if (!visited[i][j]) {
				int root = encode(i, j);
				connectedComponents[root] = root;
				Node& node = graph[root];
				node.color = board[i][j];
				node.size = 0;
				stack<int> s;
				s.push(root);
				while (!s.empty()) {
					++node.size;
					int element = s.top();
					s.pop();
					connectedComponents[element] = root;
					int x = element / n;
					int y = element % n;
					visited[x][y] = true;
					for (int k = 0; k < 4; ++k) {
						int xx = x + shift[k][0];
						int yy = y + shift[k][1];
						if (xx >= 0 && xx < n && yy >= 0 && yy < n && !visited[xx][yy] && board[x][y] == board[xx][yy])
							s.push(encode(xx, yy));
					}
				}
			}
}

void printConnectedComponents() {
	cout << graph.size() << endl;
	for (auto it = graph.begin(); it != graph.end(); ++it) {
		cout << "Color " << it->second.color << ": ";
		for (size_t i = 0; i < connectedComponents.size(); ++i)
			if (connectedComponents[i] == it->first)
				cout << "(" << i / n << ", " << i % n << ") ";
		cout << endl;
	}
}

void buildGraph() {
	for (int i = 0; i < n; ++i)
		for (int j = 0; j < n; ++j)
			if (visited[i][j]) {
				int root = connectedComponents[encode(i, j)];
				stack<int> s;
				s.push(root);
				while (!s.empty()) {
					int element = s.top();
					s.pop();
					int x = element / n;
					int y = element % n;
					visited[x][y] = false;
					for (int k = 0; k < 4; ++k) {
						int xx = x + shift[k][0];
						int yy = y + shift[k][1];
						if (xx >= 0 && xx < n && yy >= 0 && yy < n && visited[xx][yy])
							if (board[x][y] != board[xx][yy]) {
								int root2 = connectedComponents[encode(xx, yy)];
								graph[root].neighbours.insert(root2);
								graph[root2].neighbours.insert(root);
							}
							else
								s.push(encode(xx, yy));
					}
				}
			}
}

void printGraph() {
	cout << graph.size() << endl;
	for (auto node = graph.begin(); node != graph.end(); ++node) {
		cout << "(" << node->first / n << ", " << node->first % n << ", Color: "
			<< node->second.color << ", Size: " << node->second.size << "): ";
		for (auto rib = node->second.neighbours.begin(); rib != node->second.neighbours.end(); ++rib)
			cout << "(" << *rib / n << ", " << *rib % n << "), ";
		cout << endl;
	}
}

short chooseNextMove() {
	int bestSize = 0;
	short bestColor = -1;
	Node& node = _graph[position];
	map<short, int> colors;
	for (auto rib = node.neighbours.begin(); rib != node.neighbours.end(); ++rib) {
		Node& node2 = _graph[*rib];
		auto it = colors.find(node2.color);
		if (it != colors.end())
			it->second += node2.size;
		else
			colors[node2.color] = node2.size;
	}
	for (auto colorIter = colors.begin(); colorIter != colors.end(); ++colorIter)
		if (colorIter->second > bestSize) {
			bestSize = colorIter->second;
			bestColor = colorIter->first;
		}
	assert(bestColor >= 0 && bestColor <= n * n);
	moves.push_back(bestColor);
	return bestColor;
}

int chooseStartPosition() {
	int bestSize = 0;
	int position = -1;
	for (auto node = graph.begin(); node != graph.end(); ++node)
		if (node->second.size > bestSize &&
			choosedStartPositions.find(node->first) == choosedStartPositions.end()) {
			bestSize = node->second.size;
			position = node->first;
		}
	choosedStartPositions.insert(position);
	return position;
}

void mergeNodes(int n1, int n2) {
	Node& node1 = _graph[n1];
	node1.neighbours.erase(n2);
	const Node& node2 = _graph[n2];
	node1.size += node2.size;
	for (auto rib = node2.neighbours.begin(); rib != node2.neighbours.end(); ++rib)
		if (*rib != n1) {
			node1.neighbours.insert(*rib);
			Node& node3 = _graph[*rib];
			node3.neighbours.erase(n2);
			node3.neighbours.insert(n1);
		}
	_graph.erase(n2);
}

void applyMove(short color) {
	vector<int> nodesForMerging;
	Node& node = _graph[position];
	node.color = color;
	for (auto rib = node.neighbours.begin(); rib != node.neighbours.end(); ++rib)
		if (_graph[*rib].color == color)
			nodesForMerging.push_back(*rib);
	for (size_t i = 0; i < nodesForMerging.size(); ++i)
		mergeNodes(position, nodesForMerging[i]);
}

void solve() {
	if (graph.size() <= 1) return;
	while (true) {
		clock_t startTime = clock();
		if (startTime > CLOCKS_PER_SEC - 3 * maxTime)
			break;
		position = chooseStartPosition();
		moves.clear();
		if (position == -1) break;
		_graph = graph;
		while (_graph.size() > 1)
			applyMove(chooseNextMove());
		if (moves.size() < bestMoves.size() || bestMoves.size() == 0) {
			bestPosition = position;
			bestMoves = moves;
		}
		clock_t time = clock() - startTime;
		if (time > maxTime) maxTime = time;
	}
}

void writeOutput() {
	FILE* fout = fopen("flood-it.out", "w");
	fprintf(fout, "%d\n", bestMoves.size());
	fprintf(fout, "%d %d\n", bestPosition / n, bestPosition % n);
	for (size_t i = 0; i < bestMoves.size(); ++i)
		fprintf(fout, "%d\n", bestMoves[i]);
	fclose(fout);
}

int main() {
	readInput();
	findConnectedComponents();
	buildGraph();
	solve();
	writeOutput();
	return 0;
}
